import {Component, OnDestroy, OnInit} from '@angular/core';
import {FormControl, FormGroup} from '@angular/forms';
import { Observable, takeUntil} from 'rxjs';
import {map, startWith} from 'rxjs/operators';

import {DatePipe} from "@angular/common";
import {LogLevel, MyLoggerServiceService} from "../../../shared/service/my-logger-service.service";

import {HttpClient} from "@angular/common/http";
import {
  ClientControllerService, HostControllerService, ReadingPayload, SensorControllerService,
  Station,
  StationControllerService, ThreePhaseDendrometer
} from "../../../../../projects/generated-api/src";
import {UnsubscribeOnDestroy} from "../../../shared/unsubscribeOnDestroy.class";
import SensorErrorEnum = ReadingPayload.SensorErrorEnum;

@Component({
  selector: 'app-pro-view',
  templateUrl: './pro-view.component.html',
  styleUrls: ['./pro-view.component.scss'],
  providers: [DatePipe]
})
export class ProViewComponent extends UnsubscribeOnDestroy implements OnInit,OnDestroy{

  //Naming the chart for Chart.js to be able to find it
  chartId_dendrometer:string="dendrometer"
  chartId_chartAll:string="chartAll"

  //Form Controls
  stationControl = new FormControl('');
  sensorControl = new FormControl();
  hostClientControl = new FormControl();
  range = new FormGroup({
    start: new FormControl<Date | null>(null),
    end: new FormControl<Date | null>(null),
  })

  //Variables to Store Data
  availableStation: string[] = ['no Station Available'];
  availableSensors: string[] = [];
  filteredStation!: Observable<string[]>;
  availableHostClientsO:checkHostsStation[]=[]
  isDendrometer:boolean=false;
  threePhaseDendromete!:ThreePhaseDendrometer
  sensorData:SensorData[]=[]
  sensorDataArray:[SensorData[]]=[[]]
  station!: Observable<Array<Station>>;

  sensorErrorEnum:SensorErrorEnum=SensorErrorEnum.NOERROR

  constructor(private stationControllerService: StationControllerService,private hostControllerService:HostControllerService,
              private clientControllerService:ClientControllerService,
              private sensorControllerService:SensorControllerService,
              private datePipe: DatePipe,
              private logger:MyLoggerServiceService,
              private http:HttpClient
              ) {
    super()  //have to call super() to call the constructor of the parent class
    this.logger.setLevel(LogLevel.LOG)
    this.logger.setTopic("ProViewInputParent")
  }

  ngOnInit() {
    //GET all available stations
    this.stationControllerService.getAll().pipe(takeUntil(this.unsubscribe$))    //unsubscribe from all subscriptions when component is destroyed
      .subscribe({
        next: data => {   //data assign to availableStation
          this.logger.info('available station', data);
          this.availableStation = data.map(x => x.id !== undefined ? x.id : "No station avalable");
        },
        complete: () => {  //when complete, create a filter for the station input
          this.filteredStation = this.stationControl.valueChanges.pipe(
            startWith(''),
            map(value => this._filterS(this.availableStation, value || '')),
          );}
      });

    //Subscribe: to Date Input ,if any change call onSelectionChange2
    this.range.valueChanges.pipe(takeUntil(this.unsubscribe$)).subscribe(() => {
      this.onSelectionChangeSensor()
    })

    //Subscribe: to Station Input ,if any change call API to get all available clientsHosts for this station
    this.stationControl.valueChanges.pipe(takeUntil(this.unsubscribe$)).subscribe(
      x=> {
                  //GET all available HostsClients for this station
                  this.stationControllerService.getById(x ?? "").subscribe({
                    next: (response) => {
                      response.clients?.forEach((client) =>
                        this.availableHostClientsO.push({ isHost: false, value: client })
                      );
                    },
                    error: () => {
                      this.logger.info("StationInput error", this.availableHostClientsO = []);
                    },
                  });

                this.logger.debug("StationInput new value", x)
                this.logger.debug("Available HostClients", this.availableHostClientsO)


              })
        }

        //**********************Helper-FUNCTIONS********************************************************************************************************************
        //create a filter for the station input field, "a" => "a","ab" ,"abc"
        private _filterS(opt:string[],value: string): string[] {
          const filterValue = value.toLowerCase();  //current value of the input field to lower case
          return opt.filter(option => option.toLowerCase().includes(filterValue)); //return all options that include the current value
        }

        convertToCSV(data: any[]){
          const rows = [];

          // Create the header row
          //[{timestamp:timestamp, sensorname1:value1,sensorname2:value2},{timestamp:12:01, Tsoil_12:23,Temp_2:31},...]
          //=>Just get the keys timestamp,sensorname1,sensorname2      ,timestamp, Tsoil_12,Temp_2
          const header = Object.keys(data[0]).join(',');
          rows.push(header);

          // Create the data rows
          //=>Just get the values timestamp,value1,value2      ,timestamp, 23,31
          data.forEach(item => {
            const row = Object.values(item).join(',');
            rows.push(row);
          });
          const csv = rows.join('\n');

          return csv;
        }

        downloadCSV() {
          // Use the first timestamp in each array of sensorDataArray as the timestamp for each rows [12:00, 12:01, 12:02, ...]
          const timestamps = this.sensorDataArray.at(1)?.map(data => data.timestamp?? '')??[];
          this.logger.debug("timestamp csv4",timestamps)
          const sensorData: any[] = [];
          //Get the selected sensorData and map it to the csv format [],[{timestamp:timestamp, sensorname:value}],[{timestamp:12:01, sensorname:Tsoil_12}]
          this.sensorDataArray.forEach((sD)=>{
           sensorData.push(   sD.map(data => ({
              timestamp: data.timestamp,
             [data.sensorName]: data.readings.map(reading => reading.value),
            })) );
          })
          this.logger.debug("sensodata csv",sensorData)
          //Build it together 1) add timestamp each row [{timestamp:12:00},{timestamp:12:01},{timestamp:12:02},...]
          let transformedData = timestamps.map((timestamp) => ({
            timestamp: timestamp
          }));
          //Build it together 2) Merge each row timestamp + ...sensorname:value [{timestamp:timestamp, sensorname:value,sensorname:value},{timestamp:12:01, Tsoil_12:23,Temp_2:31},...]
          this.logger.debug("transformedData/merged timestamp",transformedData)
          for (let i = 1; i < sensorData.length; i++) {
            transformedData= transformedData.map((obj, ii) => {
              return { ...obj, ...sensorData[i][ii] };
            });
          }
          this.logger.debug("transformedData/merged timestamo<->Sensordata",transformedData)
          const csv = this.convertToCSV(transformedData);
          const blob = new Blob([csv], { type: 'text/csv' });
          const url = window.URL.createObjectURL(blob);
          // Create a link element and click it to trigger the download
          const a = document.createElement('a');
          a.style.display = 'none';
          a.href = url;
          a.download = 'forte-sensor-data.csv';
          document.body.appendChild(a);
          a.click();
          document.body.removeChild(a);
        }


        onSelectionChangeClientHostInput(event: checkHostsStation[]) {
         //set all available Sensors to none
          this.availableSensors=[]

          //GET the available sensors from the selected Client/Host Input
          if (event && event.length >= 1) {
            event.forEach(x=>{
              if(x.isHost){}
              //if it is not a host it has to be a client
              else{
                this.clientControllerService.getClientById(x.value??'').subscribe(
                  s=>{
                    //save the available sensors in an array which will be displayed in the sensorInput
                    this.availableSensors=[...this.availableSensors,...s.sensors??""]
                  }
                )
              }
              this.logger.info("Client/Host Input all available Sensors",this.availableSensors)
            })
          }
        }

        onSelectionChangeSensor() {
          //set all data for each selection to emtpy
          this.sensorData=[]
          this.sensorDataArray=[[]]
          let sensor_dend:String="ddm_id_1"
          //convert the inputDate to Date for API
          let datePipe = new DatePipe('en-US');
          let dateStart = datePipe.transform(this.range.value.start, 'dd-MM-yyyyTHH:mm:ss');
          let dateEnd = datePipe.transform(this.range.value.end, 'dd-MM-yyyyTHH:mm:ss');
          this.logger.info("DatenInptut",dateStart,"to",dateEnd)
          this.isDendrometer=false;
          //just call the api if at least 1 client is selected
            if (this.sensorControl.value && this.sensorControl.value.length >= 1) {
              console.log("sensors",this.sensorControl.value)
              let s_id:String=this.sensorControl.value[0]
              this.sensorControllerService.getSensorById(s_id+"").subscribe(
                s=>{
                  this.logger.info("GET sensor type",s.measurements?.at(0)?.type)
                  if (s.measurements?.at(0)?.type=="CIRCUMFERENCE_INCREMENT"){
                    this.isDendrometer=true
                    sensor_dend=s_id
                    this.http.get("http://localhost:8080/api/v1/server/get/dendrometer/range?"+
                      "sensorId="+sensor_dend+
                      "&from="+dateStart+
                      "&to="+dateEnd).subscribe(
                      // @ts-ignore
                      x=>this.logger.info("dendrometer values",this.threePhaseDendromete=x))
                  }
                })
              console.log("s_id",s_id)
              //save the data from the API in sensorDataArry=>passed to child element + used for visualisation
              this.sensorControl.value.forEach((sensor:string)=>
               // this.serverControllerService.getReadingsBySensorIdAndTimeRange(sensor, dateStart + "", dateEnd + "").subscribe(
                this.http.get("http://localhost:8080/api/v1/server/get/sensor/range?sensorId="+sensor
                  +"&from="+dateStart
                  +"&to="+ dateEnd).subscribe({
                  next: (response) => {
                    console.log(response)
                    // @ts-ignore
                    this.sensorData = response
                    this.sensorDataArray.push(this.sensorData)
                    this.logger.info("GET array sensor Data", this.sensorDataArray)
                  },
                  error: () => this.logger.error("Get sensordata error")
            })
              )
            }
        }
}

//Interface for Data to work with before downloading
export interface checkHostsStation{
  isHost:boolean
  value:string
}

export interface SensorData {
  _id: string;
  readings: { name: string, value: number }[];
  sensorName: string;
  timestamp: string;
  uniqueHardwareName: string;
  errorStatus: ReadingPayload.SensorErrorEnum;
}

